''' This is the code of my Personal Product. It is throughly commented, like
annotations in a book these comments explain the purpose and reason of various
parts of the code.'''

'''Background music is "Be-Bop-A-Lula" by Gene Vincent and His Blue Caps. (1956)
from www.simplythebest.net'''


# These lines allow me to use "modules" which are other programs I can use to
#   save work by using already written functions and definitions.
import pygame, random, sys
from pygame.locals import *

# These are variables. They are properties such as the size of the window.
WINDOWWIDTH = 600
WINDOWHEIGHT = 600
TEXTCOLOR = (255, 255, 255)
BACKGROUNDCOLOR = (0, 0, 0)
FPS = 40
BLOCKMINSIZE = 10
BLOCKMAXSIZE = 40
BLOCKMINSPEED = 1
BLOCKMAXSPEED = 8
ADDNEWBLOCKRATE = 6
PLAYERMOVERATE = 5

# This is a function that allows the program to exit smoothly.
def terminate():
    pygame.quit()
    sys.exit()

# like this function's name suggests it waits for the player to press a key
def waitForPlayerToPressKey():
    while True:
        for event in pygame.event.get():
            if event.type == QUIT:
                terminate()
            if event.type == KEYDOWN:
                if event.key == K_ESCAPE: # pressing escape quits
                    terminate()
                return

#This is the function that determines whether the player has hit a block.
def playerHasHitBLOCK(playerRect, BLOCKs):
    for b in BLOCKs:
        if playerRect.colliderect(b['rect']): # This says that if a player has
            return True                       # hit a block then return a value
    return False                              # of true to the function.

# This is the function that draws text.
def drawText(text, font, surface, x, y):
    textobj = font.render(text, 1, TEXTCOLOR)
    textrect = textobj.get_rect()
    textrect.topleft = (x, y)
    surface.blit(textobj, textrect)

# set up pygame, the window, and the mouse cursor
pygame.init()
mainClock = pygame.time.Clock()
windowSurface = pygame.display.set_mode((WINDOWWIDTH, WINDOWHEIGHT))
pygame.display.set_caption('Personal Product')
pygame.mouse.set_visible(False)

# This variable sets the default font.
font = pygame.font.SysFont(None, 48)

# This sections sets up the sounds.
winSound = pygame.mixer.Sound("Resources/win.mp3")
gameOverSound = pygame.mixer.Sound('Resources/gameover.wav')
pygame.mixer.music.load('Resources/background.mid')

# set up images
playerImage = pygame.image.load('Resources/player.png')
playerRect = playerImage.get_rect()
BLOCKImage = pygame.image.load('Resources/BLOCK.png')

# This is the start screen.
drawText('My Personal Product', font, windowSurface, 150, 200)
drawText('Press a key to start.', font, windowSurface, (WINDOWWIDTH / 3) - 30,
         (WINDOWHEIGHT / 3) + 50)
pygame.display.update()
waitForPlayerToPressKey()


topScore = 0
while True:
    # set up the start of the game
    BLOCKs = []
    score = 0
    playerRect.topleft = (WINDOWWIDTH / 2, WINDOWHEIGHT - 50)
    moveLeft = moveRight = moveUp = moveDown = False
    reverseCheat = slowCheat = False
    BLOCKAddCounter = 0
    pygame.mixer.music.play(-1, 0.0)

    while True: # the game loop runs while the game part is playing
        score += 1 # increase score

        for event in pygame.event.get():
            if event.type == QUIT:
                terminate()

            if event.type == KEYDOWN:
                if event.key == ord('z'):
                    reverseCheat = True
                if event.key == ord('x'):
                    slowCheat = True
                if event.key == K_LEFT or event.key == ord('a'):
                    moveRight = False
                    moveLeft = True
                if event.key == K_RIGHT or event.key == ord('d'):
                    moveLeft = False
                    moveRight = True
                if event.key == K_UP or event.key == ord('w'):
                    moveDown = False
                    moveUp = True
                if event.key == K_DOWN or event.key == ord('s'):
                    moveUp = False
                    moveDown = True

            if event.type == KEYUP:
                if event.key == ord('z'):
                    reverseCheat = False
                    score = 0
                if event.key == ord('x'):
                    slowCheat = False
                    score = 0
                if event.key == K_ESCAPE:
                        terminate()

                if event.key == K_LEFT or event.key == ord('a'):
                    moveLeft = False
                if event.key == K_RIGHT or event.key == ord('d'):
                    moveRight = False
                if event.key == K_UP or event.key == ord('w'):
                    moveUp = False
                if event.key == K_DOWN or event.key == ord('s'):
                    moveDown = False

            if event.type == MOUSEMOTION:
                # If the mouse moves, move the player where the cursor is.
                playerRect.move_ip(event.pos[0] - playerRect.centerx, event.pos[1] - playerRect.centery)

        # Add new BLOCKs at the top of the screen, if needed.
        if not reverseCheat and not slowCheat:
            BLOCKAddCounter += 1
        if BLOCKAddCounter == ADDNEWBLOCKRATE:
            BLOCKAddCounter = 0
            BLOCKSize = random.randint(BLOCKMINSIZE, BLOCKMAXSIZE)
            newBLOCK = {'rect': pygame.Rect(random.randint(0, WINDOWWIDTH-BLOCKSize), 0 - BLOCKSize, BLOCKSize, BLOCKSize),
                        'speed': random.randint(BLOCKMINSPEED, BLOCKMAXSPEED),
                        'surface':pygame.transform.scale(BLOCKImage, (BLOCKSize, BLOCKSize)),
                        }

            BLOCKs.append(newBLOCK)

        # Move the player around.
        if moveLeft and playerRect.left > 0:
            playerRect.move_ip(-1 * PLAYERMOVERATE, 0)
        if moveRight and playerRect.right < WINDOWWIDTH:
            playerRect.move_ip(PLAYERMOVERATE, 0)
        if moveUp and playerRect.top > 0:
            playerRect.move_ip(0, -1 * PLAYERMOVERATE)
        if moveDown and playerRect.bottom < WINDOWHEIGHT:
            playerRect.move_ip(0, PLAYERMOVERATE)

        # Move the mouse cursor to match the player.
        pygame.mouse.set_pos(playerRect.centerx, playerRect.centery)

        # Move the BLOCKs down.
        for b in BLOCKs:
            if not reverseCheat and not slowCheat:
                b['rect'].move_ip(0, b['speed'])
            elif reverseCheat:
                b['rect'].move_ip(0, -5)
            elif slowCheat:
                b['rect'].move_ip(0, 1)

         # Delete BLOCKs that have fallen past the bottom.
        for b in BLOCKs[:]:
            if b['rect'].top > WINDOWHEIGHT:
                BLOCKs.remove(b)

        # Draw the game world on the window.
        windowSurface.fill(BACKGROUNDCOLOR)

        # Draw the score and top score.
        drawText('Score: %s' % (score), font, windowSurface, 10, 0)
        drawText('Top Score: %s' % (topScore), font, windowSurface, 10, 40)

        # Draw the player's rectangle
        windowSurface.blit(playerImage, playerRect)

        # Draw each BLOCK
        for b in BLOCKs:
            windowSurface.blit(b['surface'], b['rect'])

        pygame.display.update()

        # Check if any of the BLOCKs have hit the player.
        if playerHasHitBLOCK(playerRect, BLOCKs):
            if score > topScore:
                topScore = score # set new top score
            break

        if score >= 3000:
            pygame.mixer.music.stop()
            drawText("CONGRATULATIONS!", font, windowSurface, 150, 300)
            drawText("YOU WIN!", font, windowSurface, 200, 350)
            pygame.mixer.music.stop()
            winSound.play()
            pygame.display.update()
            waitForPlayerToPressKey()
            

        mainClock.tick(FPS)

    # Stop the game and show the "Game Over" screen.
    pygame.mixer.music.stop()
    gameOverSound.play()

    drawText('GAME OVER', font, windowSurface, (WINDOWWIDTH / 3), (WINDOWHEIGHT / 3))
    drawText('Press a key to play again.', font, windowSurface, (WINDOWWIDTH / 3) - 80, (WINDOWHEIGHT / 3) + 50)
    pygame.display.update()
    waitForPlayerToPressKey()

    gameOverSound.stop()
